@@ -2,6 +2,8 @@ package DateTime::Duration;
 
 use strict;
 
+use Carp ();
+use DateTime::Helpers;
 use Params::Validate qw( validate SCALAR );
 
 use overload ( fallback => 1,
@@ -16,6 +18,8 @@ use constant MAX_NANOSECONDS => 1_000_000_000;  # 1E9 = almost 32 bits
 
 my @all_units = qw( months days minutes seconds nanoseconds );
 
+# XXX - need to reject non-integers but accept infinity, NaN, &
+# 1.56e+18
 sub new
 {
     my $class = shift;
@@ -152,6 +156,7 @@ sub in_units
         $ret{hours} = int( $minutes / 60 );
         $minutes -= $ret{hours} * 60;
     }
+
     if ( $units{minutes} )
     {
         $ret{minutes} = $minutes
@@ -175,6 +180,24 @@ sub is_wrap_mode     { $_[0]->{end_of_month} eq 'wrap'   ? 1 : 0 }
 sub is_limit_mode    { $_[0]->{end_of_month} eq 'limit'  ? 1 : 0 }
 sub is_preserve_mode { $_[0]->{end_of_month} eq 'preserve' ? 1 : 0 }
 
+sub end_of_month_mode { $_[0]->{end_of_month} }
+
+sub calendar_duration
+{
+    my $self = shift;
+
+    return
+        (ref $self)->new( map { $_ => $self->{$_} } qw( months days end_of_month ) )
+}
+
+sub clock_duration
+{
+    my $self = shift;
+
+    return
+        (ref $self)->new( map { $_ => $self->{$_} } qw( minutes seconds nanoseconds end_of_month ) )
+}
+
 sub inverse
 {
     my $self = shift;
@@ -251,7 +274,7 @@ sub _add_overload
 
     ($d1, $d2) = ($d2, $d1) if $rev;
 
-    if ( UNIVERSAL::isa( $d2, 'DateTime' ) )
+    if ( DateTime::Helpers::isa( $d2, 'DateTime' ) )
     {
         $d2->add_duration($d1);
         return;
@@ -267,8 +290,8 @@ sub _subtract_overload
 
     ($d1, $d2) = ($d2, $d1) if $rev;
 
-    die "Cannot subtract a DateTime object from a DateTime::Duration object"
-        if UNIVERSAL::isa( $d2, 'DateTime' );
+    Carp::croak( "Cannot subtract a DateTime object from a DateTime::Duration object" )
+        if DateTime::Helpers::isa( $d2, 'DateTime' );
 
     return $d1->clone->subtract_duration($d2);
 }
@@ -284,7 +307,8 @@ sub _multiply_overload
 
 sub _compare_overload
 {
-    die "DateTime::Duration does not overload comparison.  See the documentation on the compare() method for details.";
+    Carp::croak( 'DateTime::Duration does not overload comparison.'
+                 . '  See the documentation on the compare() method for details.' );
 }
 
 
@@ -312,20 +336,6 @@ DateTime::Duration - Duration objects for date math
   # Convert to different units
   $d->in_units('days', 'hours', 'seconds');
 
-  # Human-readable accessors, always positive
-  $d->years;
-  $d->months;
-  $d->weeks;
-  $d->days;
-  $d->hours;
-  $d->minutes;
-  $d->seconds;
-  $d->nanoseconds;
-
-  if ( $d->is_positive ) { ... }
-  if ( $d->is_zero )     { ... }
-  if ( $d->is_negative ) { ... }
-
   # The important parts for date math
   $d->delta_months
   $d->delta_days
@@ -339,6 +349,8 @@ DateTime::Duration - Duration objects for date math
   $d->is_limit_mode
   $d->is_preserve_mode
 
+  print $d->end_of_month_mode;
+
   # Multiple all deltas by -1
   my $opposite = $d->inverse;
 
@@ -350,6 +362,21 @@ DateTime::Duration - Duration objects for date math
   my @sorted =
       sort { DateTime::Duration->compare( $a, $b, $base_dt ) } @durations;
 
+  # Human-readable accessors, always positive, but use
+  # DateTime::Format::Duration instead
+  $d->years;
+  $d->months;
+  $d->weeks;
+  $d->days;
+  $d->hours;
+  $d->minutes;
+  $d->seconds;
+  $d->nanoseconds;
+
+  if ( $d->is_positive ) { ... }
+  if ( $d->is_zero )     { ... }
+  if ( $d->is_negative ) { ... }
+
 =head1 DESCRIPTION
 
 This is a simple class for representing duration objects.  These
@@ -378,22 +405,24 @@ This method takes the parameters "years", "months", "weeks", "days",
 of these except "end_of_month" are numbers.  If any of the numbers are
 negative, the entire duration is negative.
 
+All of the numbers B<must be integers>.
+
 Internally, years as just treated as 12 months.  Similarly, weeks are
 treated as 7 days, and hours are converted to minutes.  Seconds and
 nanoseconds are both treated separately.
 
 The "end_of_month" parameter must be either "wrap", "limit", or
-"preserve".  These specify how changes across the end of a month are
-handled.
+"preserve".  This parameter specifies how date math that crosses the
+end of a month is handled.
 
 In "wrap" mode, adding months or years that result in days beyond the
 end of the new month will roll over into the following month.  For
 instance, adding one year to Feb 29 will result in Mar 1.
 
-If you specify "end_of_month" mode as "limit", the end of the month
-is never crossed.  Thus, adding one year to Feb 29, 2000 will result
-in Feb 28, 2001.  However, adding three more years will result in Feb
-28, 2004, not Feb 29.
+If you specify "end_of_month" mode as "limit", the end of the month is
+never crossed.  Thus, adding one year to Feb 29, 2000 will result in
+Feb 28, 2001.  If you were to then add three more years this will
+result in Feb 28, 2004.
 
 If you specify "end_of_month" mode as "preserve", the same calculation
 is done as for "limit" except that if the original date is at the end
@@ -421,6 +450,28 @@ after taking away the larger units given, so for example:
   $dur->in_units( 'years' );            # 2
   $dur->in_units( 'months' );           # 27
   $dur->in_units( 'years', 'months' );  # (2, 3)
+  $dur->in_units( 'weeks', 'days' );    # (0, 0) !
+
+
+The last example demonstrates that there will not be any conversion
+between units which don't have a fixed conversion rate.  The only
+conversions possible are:
+
+=over 8
+
+=item * year <=> months
+
+=item * weeks <=> days
+
+=item * hours <=> minutes
+
+=item * seconds <=> nanoseconds
+
+=back
+
+For the explanation of why this happens, please see the L<How Date
+Math is Done|DateTime/"How Date Math is Done"> section of the
+DateTime.pm documentation
 
 Note that the numbers returned by this method may not match the values
 given to the constructor.
@@ -432,27 +483,6 @@ still computes in terms of all given units).
 If you need more flexibility in presenting information about
 durations, please take a look a C<DateTime::Format::Duration>.
 
-=item * years, months, weeks, days, hours, minutes, seconds, nanoseconds
-
-These methods return numbers indicating how many of the given unit the
-object represents, after having done a conversion to any larger units.
-For example, days are first converted to weeks, and then the remainder
-is returned.  These numbers are always positive.
-
-Here's what each method returns:
-
- $dur->year()    == abs( $dur->in_units('years') )
- $dur->months()  == ( abs( $dur->in_units( 'months', 'years' ) ) )[0]
- $dur->weeks()   == abs( $dur->in_units( 'weeks' ) )
- $dur->days()    == ( abs( $dur->in_units( 'days', 'weeks' ) ) )[0]
- $dur->hours()   == abs( $dur->in_units( 'hours' ) )
- $dur->minutes   == ( abs( $dur->in_units( 'minutes', 'hours' ) ) )[0]
- $dur->seconds   == abs( $dur->in_units( 'seconds' ) )
- $dur->nanoseconds() == abs( $dur->in_units( 'nanoseconds', 'seconds' ) )
-
-If this seems confusing, remember that you can always use the
-C<in_units()> method to specify exactly what you want.
-
 =item * delta_months, delta_days, delta_minutes, delta_seconds, delta_nanoseconds
 
 These methods provide the information C<DateTime.pm> needs for doing
@@ -475,6 +505,20 @@ will return false for B<all> of these methods.
 
 Indicates what mode is used for end of month wrapping.
 
+=item * end_of_month_mode
+
+Returns one of "wrap", "limit", or "preserve".
+
+=item * calendar_duration
+
+Returns a new object with the same I<calendar> delta (months and days
+only) and end of month mode as the current object.
+
+=item * clock_duration
+
+Returns a new object with the same I<clock> deltas (minutes, seconds,
+and nanoseconds) and end of month mode as the current object.
+
 =item * inverse
 
 Returns a new object with the same deltas as the current object, but
@@ -511,15 +555,40 @@ if used to compare two duration objects containing different units.
 It will also give non-repeatable results if the durations contain
 multiple types of units, such as months and days.
 
-However, if you know that both objects only contain the same units,
-and just a single I<type>, then the results of the comparison will be
+However, if you know that both objects only consist of one type of
+unit (months I<or> days I<or> hours, etc.), and each duration contains
+the same type of unit, then the results of the comparison will be
 repeatable.
 
+=item * years, months, weeks, days, hours, minutes, seconds, nanoseconds
+
+These methods return numbers indicating how many of the given unit the
+object represents, after having done a conversion to any larger units.
+For example, days are first converted to weeks, and then the remainder
+is returned.  These numbers are always positive.
+
+Here's what each method returns:
+
+ $dur->year()    == abs( $dur->in_units('years') )
+ $dur->months()  == ( abs( $dur->in_units( 'months', 'years' ) ) )[0]
+ $dur->weeks()   == abs( $dur->in_units( 'weeks' ) )
+ $dur->days()    == ( abs( $dur->in_units( 'days', 'weeks' ) ) )[0]
+ $dur->hours()   == abs( $dur->in_units( 'hours' ) )
+ $dur->minutes   == ( abs( $dur->in_units( 'minutes', 'hours' ) ) )[0]
+ $dur->seconds   == abs( $dur->in_units( 'seconds' ) )
+ $dur->nanoseconds() == abs( $dur->in_units( 'nanoseconds', 'seconds' ) )
+
+If this seems confusing, remember that you can always use the
+C<in_units()> method to specify exactly what you want.
+
+Better yet, if you are trying to generate output suitable for humans,
+use the C<DateTime::Format::Duration> module.
+
 =back
 
 =head2 Overloading
 
-This class overload addition, subtraction, and mutiplication.
+This class overloads addition, subtraction, and mutiplication.
 
 Comparison is B<not> overloaded.  If you attempt to compare durations
 using C<< <=> >> or C<cmp>, then an exception will be thrown!  Use the
@@ -539,9 +608,9 @@ stole all the code from.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003 David Rolsky.  All rights reserved.  This program
-is free software; you can redistribute it and/or modify it under the
-same terms as Perl itself.
+Copyright (c) 2003-2006 David Rolsky.  All rights reserved.  This
+program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
 Portions of the code in this distribution are derived from other
 works.  Please see the CREDITS file for more details.
